package com.bonitasoft.custompage.bigApp.configurationAnalysis;

import org.bonitasoft.engine.api.IdentityAPI;
import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.log.event.BEvent;
import org.bonitasoft.log.event.BEventFactory;
import org.bonitasoft.serverconfiguration.CollectOperation;
import org.bonitasoft.serverconfiguration.CollectResult;
import org.bonitasoft.serverconfiguration.CollectResultDecoMap;
import org.bonitasoft.serverconfiguration.ConfigAPI;
import org.bonitasoft.serverconfiguration.referentiel.BonitaConfigPath;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

public class ConfigurationAnalysis {

    /**
     * collect all properties information
     * @param parameters
     * @param pageDirectory
     * @return
     */
    public static Map<String, Object> propsCollects(Object parameters, File pageDirectory, ProcessAPI processAPI, IdentityAPI identityAPI, long tenantId) {
        CollectOperation.BonitaAccessor bonitaAccessor = new CollectOperation.BonitaAccessor();
        bonitaAccessor.processAPI = processAPI;
        bonitaAccessor.identityAPI = identityAPI;
        bonitaAccessor.tenantId = tenantId;

        File fileBundle = null;

        if (System.getProperty( "catalina.home" ) != null && !System.getProperty( "catalina.home" ).isEmpty()) {
            fileBundle = new File(System.getProperty( "catalina.home" ) + "/logs/");
        } else {
            fileBundle = new File(System.getProperty( "jboss.home.dir" )).getParentFile();
        }

        ArrayList<BEvent> listEvents = new ArrayList<>();

        try {
            fileBundle = new File(fileBundle.getCanonicalPath());
        } catch (Exception e) {
        }

        ConfigAPI.CollectParameter collectParameter = getInstanceFromObject(parameters);
        BonitaConfigPath localBonitaConfig;
        ConfigAPI currentConfig;
        if (collectParameter.useLocalFile) {
            localBonitaConfig = BonitaConfigPath.getInstance(fileBundle);
            currentConfig = ConfigAPI.getInstance( localBonitaConfig );
            collectParameter.localFile = fileBundle;
            if (collectParameter.doSetupPull)
                listEvents.addAll( currentConfig.setupPull() );
        }
        else {
            localBonitaConfig = BonitaConfigPath.getInstance(collectParameter.localFile);
            currentConfig = ConfigAPI.getInstance( localBonitaConfig );
        }

        // now, collect result
        // Note : collectParameters contains the marker "hidePassword", to pilot the collect. In case of analysis (and only for analysis), this marker is set to false
        CollectResult collectResult = currentConfig.collectParameters( collectParameter, CollectResult.COLLECTLOGSTRATEGY.LOGALL, bonitaAccessor);

        // I want the result in JSON, so use a ResultDecoMap
        CollectResultDecoMap decoMap = new CollectResultDecoMap(collectResult, "", localBonitaConfig.getRootPath() );
        decoMap.setLineFeedToHtml( true );
        // ok, get the value of decoration

        Map<String,Object> result=new HashMap<>();

        result.put("analysis", decoMap.getMap( CollectOperation.TYPECOLLECT.ANALYSIS));

        // collect errors
        listEvents.addAll( collectResult.getErrors());
        //result.put("listevents", BEventFactory.getSyntheticHtml(listEvents));

        return result;

    }

    public static String getConfigurationAnalysisInfosExport(Object parameters, File pageDirectory, ProcessAPI processAPI, IdentityAPI identityAPI, long tenantId) {
        return propsCollects( parameters, pageDirectory, processAPI,identityAPI, tenantId ).toString();
    }

    public static ConfigAPI.CollectParameter getInstanceFromObject(Object parameters) {
        ConfigAPI.CollectParameter comparaisonParameter = new ConfigAPI.CollectParameter();

        comparaisonParameter.listTypeCollect = new HashSet();

        comparaisonParameter.listTypeCollect.add( CollectOperation.TYPECOLLECT.ANALYSIS);

        comparaisonParameter.hidePassword = false;

        comparaisonParameter.useLocalFile = true;

        //comparaisonParameter.localFile = new File( "test" );

        return comparaisonParameter;
    }
}
